@@ -113,6 +113,15 @@ namespace Oxygen
         local = other;
     }
 
+    //_______________________________________________________
+    void ShadowCache::setShadowSize( QPalette::ColorGroup group, qreal size )
+    {
+        ShadowConfiguration& local = (group == QPalette::Active ) ? _activeShadowConfiguration:_inactiveShadowConfiguration;
+        if( local.shadowSize() == size ) return;
+        local.setShadowSize( size );
+        invalidateCaches();
+    }
+
     //_______________________________________________________
     TileSet* ShadowCache::tileSet( const Key& key )
     {
@@ -242,16 +251,15 @@ namespace Oxygen
 
                 // outer (spread) gradient
                 const qreal gradientSize = shadowSize;
-                const qreal hoffset = shadowConfiguration.horizontalOffset()*gradientSize/fixedSize;
                 const qreal voffset = shadowConfiguration.verticalOffset()*gradientSize/fixedSize;
 
-                QRadialGradient rg = QRadialGradient( size+12.0*hoffset, size+12.0*voffset, gradientSize );
+                QRadialGradient rg = QRadialGradient( size, size+12.0*voffset, gradientSize );
                 rg.setColorAt(1, Qt::transparent );
 
                 // gaussian shadow is used
                 int nPoints( (10*gradientSize)/fixedSize );
                 Gaussian f( 0.46, 0.34 );
-                QColor c = shadowConfiguration.outerColor();
+                QColor c = shadowConfiguration.useOuterColor() ? shadowConfiguration.outerColor():shadowConfiguration.innerColor();
                 for( int i = 0; i < nPoints; i++ )
                 {
                     qreal x = qreal(i)/nPoints;
@@ -270,16 +278,15 @@ namespace Oxygen
             {
                 // inner (sharp gradient)
                 const qreal gradientSize = qMin( shadowSize, fixedSize );
-                const qreal hoffset( 0 );
                 const qreal voffset( 0.2 );
 
-                QRadialGradient rg = QRadialGradient( size+hoffset, size+voffset, gradientSize );
+                QRadialGradient rg = QRadialGradient( size, size+voffset, gradientSize );
                 rg.setColorAt(1, Qt::transparent );
 
                 // parabolic shadow is used
                 int nPoints( (10*gradientSize)/fixedSize );
                 Parabolic f( 1.0, 0.22 );
-                QColor c = shadowConfiguration.outerColor();
+                QColor c = shadowConfiguration.useOuterColor() ? shadowConfiguration.outerColor():shadowConfiguration.innerColor();
                 for( int i = 0; i < nPoints; i++ )
                 {
                     qreal x = qreal(i)/nPoints;
@@ -298,16 +305,15 @@ namespace Oxygen
 
                 // mid gradient
                 const qreal gradientSize = qMin( shadowSize, (shadowSize+2*fixedSize)/3 );
-                const qreal hoffset = shadowConfiguration.horizontalOffset()*gradientSize/fixedSize;
                 const qreal voffset = shadowConfiguration.verticalOffset()*gradientSize/fixedSize;
 
                 // gaussian shadow is used
-                QRadialGradient rg = QRadialGradient( size+8.0*hoffset, size+8.0*voffset, gradientSize );
+                QRadialGradient rg = QRadialGradient( size, size+8.0*voffset, gradientSize );
                 rg.setColorAt(1, Qt::transparent );
 
                 int nPoints( (10*gradientSize)/fixedSize );
                 Gaussian f( 0.54, 0.21);
-                QColor c = shadowConfiguration.outerColor();
+                QColor c = shadowConfiguration.useOuterColor() ? shadowConfiguration.outerColor():shadowConfiguration.innerColor();
                 for( int i = 0; i < nPoints; i++ )
                 {
                     qreal x = qreal(i)/nPoints;
@@ -325,16 +331,15 @@ namespace Oxygen
 
                 // outer (spread) gradient
                 const qreal gradientSize = shadowSize;
-                const qreal hoffset = shadowConfiguration.horizontalOffset()*gradientSize/fixedSize;
                 const qreal voffset = shadowConfiguration.verticalOffset()*gradientSize/fixedSize;
 
                 // gaussian shadow is used
-                QRadialGradient rg = QRadialGradient( size+20.0*hoffset, size+20.0*voffset, gradientSize );
+                QRadialGradient rg = QRadialGradient( size, size+20.0*voffset, gradientSize );
                 rg.setColorAt(1, Qt::transparent );
 
                 int nPoints( (20*gradientSize)/fixedSize );
                 Gaussian f( 0.155, 0.445);
-                QColor c = shadowConfiguration.outerColor();
+                QColor c = shadowConfiguration.useOuterColor() ? shadowConfiguration.outerColor():shadowConfiguration.innerColor();
                 for( int i = 0; i < nPoints; i++ )
                 {
                     qreal x = qreal(i)/nPoints;
